#!/usr/bin/env jython
"""
Message.py - 0.7 - 2007-10-19

This module contains the Jython class JyMQMessage.
This class should provide easier access to the MQMessage object,
as derived from com.ibm.mq.jar.

Copyright (C) 2007  Gert-Jan den Besten

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>   
"""

# Java/MQ imports
from com.ibm.mq import MQException, MQC, MQMessage

# Jython imports
import jarray

# Python imports
from CCSIDs import dctCcsid2Python




class JyMQMD(object):
    """
    JyMQMD is a placeholder for MQMD fields.
    """
    def __init__(self, objMQMessage):
        """
        """
        self.StrucId          = '' 
        self.Version          = MQC.MQMD_VERSION_2                    # Integer
        self.Report           = objMQMessage.report                   # Integer
        self.MsgType          = objMQMessage.messageType              # Integer
        self.Expiry           = objMQMessage.expiry                   # Integer
        self.Feedback         = objMQMessage.feedback                 # Integer
        self.Encoding         = objMQMessage.encoding                 # Integer
        self.CodedCharSetId   = objMQMessage.characterSet             # Integer
        self.Format           = objMQMessage.format                   # String
        self.Priority         = objMQMessage.priority                 # Integer
        self.Persistence      = objMQMessage.persistence              # Integer
        self.MsgId            = objMQMessage.messageId                # Java byte array
        self.CorrelId         = objMQMessage.correlationId            # Java byte array
        self.BackoutCount     = objMQMessage.backoutCount             # Integer
        self.ReplyToQ         = objMQMessage.replyToQueueName         # String
        self.ReplyToQMgr      = objMQMessage.replyToQueueManagerName  # String
        self.UserIdentifier   = objMQMessage.userId                   # String
        self.AccountingToken  = objMQMessage.accountingToken          # Java byte array
        self.ApplIdentityData = objMQMessage.applicationIdData        # String
        self.PutApplType      = objMQMessage.putApplicationType       # Integer
        self.PutApplName      = objMQMessage.putApplicationName       # String
        self.PutDate          = ''
        self.PutTime          = ''
        self.ApplOriginData   = objMQMessage.applicationOriginData    # String
        self.GroupId          = objMQMessage.groupId                  # Java byte array
        self.MsgSeqNumber     = objMQMessage.messageSequenceNumber    # Integer
        self.Offset           = objMQMessage.offset                   # Integer
        self.MsgFlags         = objMQMessage.messageFlags             # Integer
        self.OriginalLength   = objMQMessage.originalLength           # Integer




class JyMQRFH2(object):
    """
    JyMQRFH2 is a placeholder for RFH2 fields (if any).
    """
    def __init__(self, objMQMessage):
        """
        """
        dctCharSet = {1200 : 'utf_16', 
                      13488: 'utf_16_be',
                      17584: 'utf_16_be',  
                      1208 : 'utf_8'}
        # Values of this dictionary are based on:
        #   http://publib.boulder.ibm.com/infocenter/db2luw/v9/index.jsp?topic=/com.ibm.db2.udb.apdv.embed.doc/doc/r0022549.htm
        
        objMQMessage.setDataOffset(0)

        self.StrucId         = objMQMessage.readString(4)
        self.Version         = objMQMessage.readInt4()
        self.StrucLength     = objMQMessage.readInt4()
        self.Encoding        = objMQMessage.readInt4()
        self.CodedCharSetId  = objMQMessage.readInt4()
        self.Format          = objMQMessage.readString(8)
        self.Flags           = objMQMessage.readInt4()
        self.NameValueCCSID  = objMQMessage.readInt4()
        if objMQMessage.getDataOffset() < self.StrucLength:
            self.NameValueLength = objMQMessage.readInt4()

            lstBytes = [objMQMessage.readUnsignedByte() for x in xrange(self.NameValueLength)]
            self.NameValueData = ''.join([chr(x) for x in lstBytes]).decode(dctCharSet[self.NameValueCCSID])




class JyMessageData(object):
    """
    """
    def __init__(self, objMQMessage, intMessageLength, intOffset, intCCSID):
        """
        """
        self.CCSID = intCCSID

        objMQMessage.setDataOffset(intOffset)
        intDataLength = intMessageLength - intOffset

        lstBytes = [objMQMessage.readUnsignedByte() for x in xrange(intDataLength)]
        self.UnicodeText = ''.join([chr(x) for x in lstBytes]).decode(dctCcsid2Python[self.CCSID])

        self.HexDump = u''.join([str(hex(x))[2:] for x in lstBytes])




class JyMQMessage(object):
    """
    """
    def __init__(self, objMQMessage=None):
        """
        """
        if not objMQMessage:
            objMQMessage = MQMessage()

        self.intPos           = 0    # Position of pointer to messagedata.
        self.intMessageLength = objMQMessage.getTotalMessageLength()
        self.intDataLength    = 0

        # Parsing MQMD header.
        self.MQMD = JyMQMD(objMQMessage)

        # Parse MQRFH2 header (if present).
        if self.MQMD.Format == 'MQHRF2  ':
            self.MQRFH2 = JyMQRFH2(objMQMessage)
            self.intPos = self.MQRFH2.StrucLength

        # Retrieving messagdata.
        # Trying to decide which CCSID to use....
        try:
            intCCSID = self.MQRFH2.CodedCharSetId
        except:
            intCCSID = self.MQMD.CodedCharSetId
            
        self.intDataLength = self.intMessageLength - self.intPos
        self.MessageData = JyMessageData(objMQMessage, self.intMessageLength, self.intPos, intCCSID)
