#!/usr/bin/env jython
"""
MQBTU - MQ Batch Test Utility - 1.0.0 - 2007-10-26

Copyright (C) 2007  Gert-Jan den Besten

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>   
"""

# Import Java-API's
from com.ibm.mq import MQMessage, MQException

# Jython imports
import jarray

# Python imports
import os, sys
from shutil       import copyfile
from ConfigParser import ConfigParser, NoSectionError, NoOptionError
from time         import sleep, strftime

# Custom imports
from JyModules.Log     import GetDateTimeStamp, CreateLog, WriteLog, CloseLog, CloseLogWithError
from JyModules.MQ.Misc import GetAllMessages, PutMessage, OpenQmgr, OpenQueue, ClearQueue, Mq24ByteField




MQException.log = None

cstCopyrightNotice = """
MQBTU  Copyright (C) 2007  Gert-Jan den Besten.
 This program comes with ABSOLUTELY NO WARRANTY.
 This is free software, and you are welcome to redistribute it
 under certain conditions. Please consult the accompanying
 "gpl-3.0.txt" file regarding the licensing terms.
"""
cstProgramVersion  = 'MQBTU 1.0.0'
cstIniFile = 'MQBTU.ini'
cstLogFile = 'MQBTU.log'
cstDashes = '-' * 70
cstDateTimeTestRun = strftime("_%Y%m%d_%H%M%S")
cstEndMessage = 'Press ENTER to end program'




def QuitProgram():
    """
    End program.
    """
    strDummy = raw_input(cstEndMessage)
    sys.exit(0)



def ReadConfigFile(filLog):
    """
    Parsing the Configfile returning the settings in dictionary dctSettings.
    """
    dctSettings = dict()

    # Trying to parse the ini-file.
    try:
        prsConfig = ConfigParser()
        prsConfig.readfp(file(cstIniFile))
    except:
        raise ValueError('No valid MQBTU.ini file found.')
    else:
        WriteLog(filLog, 'Start processing MQBTU.ini.')

    # Trying to retrieve all mandatory settings.
    try:
        dctSettings['qmgr']    = prsConfig.get('Main', 'qmgr')
        dctSettings['rootdir'] = prsConfig.get('Main', 'rootdir')
        dctSettings['delay']   = prsConfig.getint('Main', 'delay')
        dctSettings['host']    = prsConfig.get('MQEnvironment', 'host')
        dctSettings['channel'] = prsConfig.get('MQEnvironment', 'channel')
        dctSettings['port']    = prsConfig.getint('MQEnvironment', 'port')

        dctQueues = dict()
        dctQueues['inputqueue']  = prsConfig.get('Inputqueue', 'inputqueue')
        lstOutput = prsConfig.options('Outputqueues')
        lstOutput.sort()
        lstOutputQueues = [prsConfig.get('Outputqueues', strOption) for strOption in lstOutput]
        dctQueues['outputqueue'] = lstOutputQueues
    except (NoSectionError, NoOptionError, ValueError), objErr:
        raise ValueError('Not able to retrieve all mandatory entries from MQBTU.ini.')
    else:
        strMess = 'All mandatory entries found in MQBTU.ini.\n' + \
                  '          rootdirectory  = ' + dctSettings['rootdir']    + '\n' + \
                  '          queuemanager   = ' + dctSettings['qmgr']       + '\n' + \
                  '          delay          = ' + str(dctSettings['delay']) + ' second(s)\n' + \
                  '          hostnaam       = ' + dctSettings['host']       + '\n' + \
                  '          channel        = ' + dctSettings['channel']    + '\n' + \
                  '          port           = ' + str(dctSettings['port'])  + '\n' + \
                  '          inputqueue     = ' + dctQueues['inputqueue']   + '\n'
        for intItem in range(1, len(lstOutputQueues) + 1):
            strItem  = '0' * (2 - len(str(intItem))) + str(intItem)
            strMess += '          outputqueue %s = %s\n' % (strItem, lstOutputQueues[intItem-1])

        WriteLog(filLog, strMess)

    # Verifying mandatory settings.
    for key in dctSettings:
        if not dctSettings[key]:
            raise ValueError('Not able to retrieve all mandatory settings from MQBTU.ini.')

    for key in dctQueues:
        if not dctQueues[key]:
            raise ValueError('Not able to retrieve all queuenames from MQBTU.ini.')

    if '' in dctQueues['outputqueue']:
        raise ValueError('Not able to retrieve all queuenames from MQBTU.ini.')

    WriteLog(filLog, 'Processing MQBTU.ini finished.')

    return dctSettings, dctQueues



def CheckTestCases(dctSettings, filLog):
    """
    Checking the existence of test cases. End program if none available.
    """
    WriteLog(filLog, 'Checking test cases:')
    strInputDir = dctSettings['rootdir'] + os.sep + 'input'

    try:
        tmpLstInputFiles = os.listdir(strInputDir)
    except:
        raise EnvironmentError("Couldn't find directory 'input'!")

    # Compile list with testcases (using list-comprehension).
    lstInput = [strFile for strFile in tmpLstInputFiles if os.path.isfile(strInputDir + os.sep + strFile)]

    if not lstInput:
        strMess = 'No test cases found in "%s".' % strInputDir
        raise EnvironmentError(strMess)
    else:
        strMess = '%i test cases found in "%s".' % (len(lstInput), strInputDir)
        print strMess; WriteLog(filLog, strMess)

    return strInputDir, lstInput



def CreateOutputDirs(dctSettings, dctQueues, objQueueManager, filLog):
    """
    Creating Queue-subdirectories in the 'output'-directory.
    """
    WriteLog(filLog, 'Creating outputdirectories:')

    lstOutputQueues = dctQueues['outputqueue'][:]
    strOutputDir = dctSettings['rootdir'] + os.sep + 'output' + cstDateTimeTestRun

    for strDir in lstOutputQueues:
        strQueueDir = strOutputDir + os.sep + strDir
        os.makedirs(strQueueDir)
        WriteLog(filLog, 'Created directory "%s".' % strQueueDir)

    WriteLog(filLog, 'Created all outputdirectories.')

    return strOutputDir



def OpenAndVerifyQueue(objQueueManager, strQueueName, filLog):
    """
    Trying to open strQueueName and check this queue for unwanted messages.
    """
    try:
        objQueue = OpenQueue(objQueueManager, strQueueName)
    except:
        objQueueManager.disconnect()
        strMess = 'Opening queue %s not successful.' % strQueueName
        raise EnvironmentError(strMess)
    else:
        WriteLog(filLog, 'Opening queue %s successful.' % strQueueName)

    if objQueue.currentDepth == 0:
        WriteLog(filLog, "Queue %s is empty." % strQueueName)
    else:
        objQueueManager.disconnect()
        strMess = '%s contains messages, please clear this queue first!' % strQueueName
        raise EnvironmentError(strMess)

    return objQueue



def OpenAllOutputQueues(objQueueManager, lstOutputQueues, filLog):
    """
    Opening all outputqueues.
    """
    dctOutputQueues = dict()

    for strQueueName in lstOutputQueues:
        dctOutputQueues[strQueueName] = OpenAndVerifyQueue(objQueueManager, strQueueName, filLog)

    return dctOutputQueues



def PutTestFile(strTest, strInputDir, myPutQueue, dctQueues, filLog):
    """
    All actions necessary to put a test case on the inputqueue.
    """
    strMess = 'Processing test case: %s' % strTest
    print strMess; WriteLog(filLog, strMess)

    # Retrieve data from textfile
    strFileName = strInputDir + os.sep + strTest
    filInput = file(strFileName, 'r')
    strMessageData = filInput.read()
    filInput.close()

    # Create testmessage
    myMessage = MQMessage()
    myMessage.messageId = Mq24ByteField(strTest)
    myMessage.writeString(strMessageData)

    PutMessage(myMessage, myPutQueue)
    strMess = 'Message with MsgID "%s" has been put on %s.' % (strTest[:24], dctQueues['inputqueue'])
    WriteLog(filLog, strMess)



def ProcessOuputQueue(strQueue, objQueue, strTest, strOutputDir, filLog):
    """
    Process all messages on strQueue. Save messagedata of all messages
    - using utf8 encoding - to the appropriate output subdirectory. 
    """
    strOutputQueueDir = strOutputDir + os.sep + strQueue + os.sep
    intNumber = objQueue.currentDepth

    if intNumber == 0:
        WriteLog(filLog, 'No messages found on queue %s.' % strQueue)
    else:
        WriteLog(filLog, '%i Messages found on queue %s.' % (intNumber, strQueue))

        intCount = 0
        for myGetMessage in GetAllMessages(objQueue):
            intCount += 1

            # Retrieve messagedata.
            strMessageData = myGetMessage.MessageData.UnicodeText
            strMessageData = strMessageData.encode('utf-8')
            bteMessageId   = myGetMessage.MQMD.MsgId
            strMessageId   = bteMessageId.tostring()    # Jython-specifiek.

            # Create outputfilename.
            strCount = '0' * (2 - len(str(intCount))) + str(intCount)
            tplTest = os.path.splitext(strTest)
            strFileName = 'output_%s_%s%s.utf8' % (tplTest[0], strCount, tplTest[1])

            # Createk outputfile.
            filOutput = file(strOutputQueueDir + strFileName, 'w')
            filOutput.write(strMessageData)
            filOutput.close()

            strMess = 'File "%s" with MsgId "%s" saved in directory "%s".' % (strFileName, strMessageId, strQueue)
            WriteLog(filLog, strMess)



def ClearPutQueue(objPutQueue, strPutQueue, filLog):
    """
    Clear myPutQueue if it contains messages.
    """
    if objPutQueue.currentDepth != 0:
        strMess = 'Messages found on inputqueue %s, clearing inputqueue now!' % strPutQueue
        print strMess; WriteLog(filLog, strMess)
        ClearQueue(objPutQueue)



def main():
    print 'Processing %s started at %s' % (cstProgramVersion, GetDateTimeStamp())
    print cstCopyrightNotice

    # Program setup.
    # Create log file. 
    try:
        filLog = CreateLog(cstLogFile, cstProgramVersion)
    except:
        print 'FATAL ERROR: Cannot create log file.'
        QuitProgram()

    # Checking the testing environment.
    try:
        dctSettings, dctQueues = ReadConfigFile(filLog)
        lstOutputQueues = dctQueues['outputqueue']
        strInputDir, lstTestFiles = CheckTestCases(dctSettings, filLog)

        # Trying to connect to queuemanager en queues.
        myQueueManager = OpenQmgr(dctSettings['qmgr'], dctSettings)

        WriteLog(filLog, "Start verifying queues:")
        objInputQueue = OpenAndVerifyQueue(myQueueManager, dctQueues['inputqueue'], filLog)
        dctOutputQueues = OpenAllOutputQueues(myQueueManager, lstOutputQueues, filLog)
        WriteLog(filLog, 'All queues succesfully verified.')
    except (EnvironmentError, ValueError), objErr:
        strErr = 'FATAL ERROR: ' + str(objErr)
        print strErr; CloseLogWithError(filLog, strErr)
        QuitProgram()

    strOutputDir = CreateOutputDirs(dctSettings, dctQueues, myQueueManager, filLog)
    WriteLog(filLog, cstDashes)

    # Start processing.
    for strTest in lstTestFiles:
        PutTestFile(strTest, strInputDir, objInputQueue, dctQueues, filLog)
        sleep(dctSettings['delay'])

        for strOutputQueue in lstOutputQueues:
            ProcessOuputQueue(strOutputQueue, dctOutputQueues[strOutputQueue], strTest, strOutputDir, filLog)

        ClearPutQueue(objInputQueue, dctQueues['inputqueue'], filLog)
        WriteLog(filLog, cstDashes)

    # Cleaning up.
    objInputQueue.close()
    for queue in dctOutputQueues: dctOutputQueues[queue].close()
    myQueueManager.disconnect()

    WriteLog(filLog, 'Test run finished.')
    WriteLog(filLog, 'Disconnecting queuemanager %s.' % dctSettings['qmgr'])
    CloseLog(filLog)

    # Preserve ini- and logfiles.
    copyfile(cstIniFile, strOutputDir + os.sep + cstIniFile)
    copyfile(cstLogFile, strOutputDir + os.sep + cstLogFile)

    strDummy = raw_input(cstEndMessage)



if __name__ == "__main__":
    main()
